#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All righwcert reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
import os, sys, string, shutil, time, glob

from hwcert.test import Test
from hwcert.tags import Constants, DeviceClass
from hwcert.udevDeviceDetector import UdevDeviceDetector
from hwcert.command import Command

class AudioTest(Test):

    def __init__(self):
        Test.__init__(self, name="audio")
        self.deviceClass = DeviceClass.audio
        self.cardNumber = 0

    def plan(self, planner):
        properties = dict()
        properties['SUBSYSTEM'] = 'sound'
        properties['SOUND_INITIALIZED'] = '1'
        properties['ID_BUS'] = 'pci'
        tests = self.planWith(planner.getUdevDeviceDetector(), properties, strict=True)
        return tests

    def getRequiredRPMs(self):
        return ["alsa-utils"]

    def logSoundCardInfo(self):
        if self.udi:
            self.device = UdevDeviceDetector().getDevice(self.udi)
            if self.device:
                print "\nSound Card:"
                print self.device.dump()
                self.cardNumber = self.udi[self.udi.rfind('/')+5:]
                for infopath in glob.glob("/proc/asound/card%s/pcm*/info" % self.cardNumber):
                    print infopath
                    Command("cat %s" % infopath).echo()
                    print ""
                self.getDeviceNumber()
                return True
        # otherwise
        return False

    def getDeviceNumber(self):
        self.deviceNumber = -1
        for eldpath in glob.glob("/proc/asound/card%s/eld*" % self.cardNumber):
            # self.deviceNumber = eldpath.split('.')[1]
            file = open(eldpath)
            lines = file.read(8192).splitlines()
            file.close()
            present = 0
            eldValid = 0
            for line in lines:
                if line.startswith('monitor_present'):
                    present = int(line.split('\t')[-1])
                if line.startswith('eld_valid'):
                    eldValid = int(line.split('\t')[-1])
            # catch first output with connected monitor
            if not present or not eldValid:
                continue
            self.deviceNumber = eldpath.split('.')[1]

    def run(self):
        FAILED = 1
        PASSED = 0

        if not self.logSoundCardInfo():
            print "Error: could not log sound card information"
            return FAILED

        if self.getMode() == Constants.auto:
            print("Error: Test Mode: %s: The audio test can not be run in auto mode since audio functions must be verified by the tester." % self.getMode())
            return FAILED

        # otherwise
        print "This test plays a sound sample and records it to another file."
        print "Please use the Volume Control application and insure that the "
        print "Capture settings will record the sound."
        if not self.promptConfirm("Continue?"):
            self.result = Constants.FAIL
            return FAILED

        waveFiles=["/usr/share/sounds/alsa/Front_Right.wav",
                   "/usr/share/sounds/alsa/Front_Left.wav"]
        recordedWaveFile="./test.wav"
        print "starting recording while playing demo sound"
        if not os.system("arecord -r 44100 -D plughw:%s %s &" % (self.cardNumber, recordedWaveFile)) is 0:
            print "Error: arecord command failed"
            self.result = Constants.FAIL
            return FAILED

        for waveFile in waveFiles:
            if os.path.exists(waveFile):
                if not os.system("aplay %s -D plughw:%s" % (waveFile, self.cardNumber)) is 0:
                    print "Error: aplay command failed"
                    self.result = Constants.FAIL
                    return FAILED
                time.sleep(1)
        sys.stdout.flush()
        os.system("killall -w -9 arecord")
        if not self.promptConfirm("Did you hear the played sound?"):
            self.result = Constants.FAIL
            return FAILED

        print "playing recorded sound"
        if not os.system("aplay %s -D plughw:%s " % (recordedWaveFile, self.cardNumber)) is 0:
            print "Error: aplay command failed"
            self.result = Constants.FAIL
            return FAILED

        if not self.promptConfirm("Did you hear the recorded sound?"):
            self.result = Constants.FAIL
            return FAILED

        outputDirectory = os.path.dirname(self.outputFile)
        shutil.copy(recordedWaveFile, outputDirectory)
        print "Copied recorded sound %s to %s" % (recordedWaveFile, outputDirectory)

        self.result = Constants.PASS
        return PASSED

if __name__ == "__main__":
    test = AudioTest()
    returnValue = test.do(sys.argv)
    sys.exit(returnValue)
